#include <avr/io.h>
#include <avr/interrupt.h>
#include "rf02.h"

typedef unsigned char	u8;
typedef signed short	s16;

unsigned char test0[32]=("Dies ist ein 433MHz Test!!    \r\n");
volatile unsigned char j;

#define KEY_PIN		PIND
#define KEY1		2
#define KEY2		3
#define KEY3		4

#define LED_DDR		DDRD
#define LED_PORT	PORTD
#define LED1		5
#define LED2		6

#define REPEAT_MASK	(1<<KEY1^1<<KEY2^1<<KEY3)			// repeat: key1, key2, key3
#define REPEAT_START	50													// after 500ms
#define REPEAT_NEXT	20														// every 200ms


u8 key_state;																			// debounced key state:
																									// bit = 1: key pressed
u8 key_press;																			// key press detect

u8 key_rpt;																				// key long press and repeat

/******************************************************************************
******************** Interrupt-Routine ****************************************
******************************************************************************/

SIGNAL (SIG_OVERFLOW0)														// every 10ms
{
  static u8 ct0, ct1, rpt;
  u8 i;
	
	j++;
  TCNT0 = (u8)(s16)-(F_CPU / 1024 * 10e-3 + 0.5);	// preload for 10ms

  i = key_state ^ KEY_PIN;												// key changed ?
  ct0 = ~( ct0 & i );														// reset or count ct0
  ct1 = ct0 ^ (ct1 & i);													// reset or count ct1
  i &= ct0 & ct1;																	// count until roll over ?
  key_state ^= i;																	// then toggle debounced state
  key_press |= key_state & i;											// 0->1: key press detect

  if( (key_state & REPEAT_MASK) == 0 )						// check repeat function
     rpt = REPEAT_START;													// start delay
  if( --rpt == 0 ){
    rpt = REPEAT_NEXT;														// repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
}

/*****************************************************************************
********************* Funktionen *********************************************
*****************************************************************************/

u8 get_key_press( u8 key_mask )
{
  cli();																					// read and clear atomic !
  key_mask &= key_press;                        	// read key(s)
  key_press ^= key_mask;                        	// clear key(s)
  sei();
  return key_mask;
}

u8 get_key_rpt( u8 key_mask )
{
  cli();																					// read and clear atomic !
  key_mask &= key_rpt;                        		// read key(s)
  key_rpt ^= key_mask;                        		// clear key(s)
  sei();
  return key_mask;
}

u8 get_key_short( u8 key_mask )
{
  cli();																					// read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}

u8 get_key_long( u8 key_mask )
{
	cli();
  return get_key_press( get_key_rpt( key_mask ));
}

/******************************************************************************
************************** Main program ***************************************
******************************************************************************/

int main(void)
{
  TCCR0 = 1<<CS02^1<<CS00;												// divide by 1024
  TIMSK = 1<<TOIE0;																// enable timer interrupt

  LED_PORT = 0x00;
  LED_DDR = 1<<DDD5|1<<DDD6|1<<DDD7;							// LEDs & Piezo Ausgang, Taster Eingang
  sei();

	LED_PORT|=1<<LED2;															// LED2 fuer 1sec. einschalten
	j=0;
	while (j<100)
		asm("nop");
	LED_PORT|=0<<LED2;															// und wieder aus
	
	rf02_init();																		// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf02_setfreq(RF02FREQ(433.92));									// Sendefrequenz auf 433,92MHz einstellen
	rf02_setpower(4);																// -12dBm Ausgangangsleistung
	rf02_setmodfreq(3);															// 120kHz Frequenzshift
	rf02_setbaud(19200);														// 19200 Baud
	
	for (;;)
	{
    if( get_key_press( 1<<KEY1 ))
      LED_PORT ^= 1<<LED1;

    if( get_key_press( 1<<KEY2 ))
      LED_PORT ^= 1<<LED2;

    if( get_key_press( 1<<KEY3 ))
      LED_PORT ^= 1<<LED1|1<<LED2;

		cli();
		rf02_txdata(test0,32);												// Text senden
		sei();
		
    LED_PORT ^= 1<<LED2;													// LED2 toggeln, 1sec. warten
		j=0;
		while (j<100)
			asm("nop");
	}
}

